/*
/ Dateiname : Pollin-Spiel.c
/ Versions  : 1.0.1
/ Datum     : 23.02.2011 
/ Autor     : Pollin-Team
/ Copyright : Pollin Electronic GmbH
/             www.pollin.de
/
/
/ Mikrocontroller :
/                                   ATtiny2313
/                              ---------------------  
/                             |                     |
/                   <---PA2---|1(/RESET/dW)   VCC 20|
/           TASTER1 <---PD0---|2(RXD)   (UCSK/SCL)19|---PB7--> ISP Programmer
/           TASTER2 <---PD1---|3(TXD)    (MISO/DO)18|---PB6--> ISP Programmer
/        DISPLAY_RS <---PA1---|4(XTAL2) (./DI/SDA)17|---PB5--> ISP Programmer
/        DISPLAY_E  <---PA0---|5(XTAL1)     (OC1B)16|---PB4--> DISPLAY_Vo 
/        EIN_TASTER <---PD2---|6(CKOUT/XCK) (OC1A)15|---PB3--> DISPLAY_DB7 
/           TASTER3 <---PD3---|7(INT1)      (OC0A)14|---PB2--> DISPLAY_DB6  
/           TASTER4 <---PD4---|8(T0)        (AIN1)13|---PB1--> DISPLAY_DB5
/            SUMMER <---PD5---|9(OC0B/T1)   (AIN0)12|---PB0--> DISPLAY_DB4 
/                             |10 GND        (ICP)11|---PD6--> DISPLAY_POWER
/                              ---------------------
/
/
/
/ Fuse-Bits: --- fr internen Clock ---
/            CKSEL3..0   0100   (1 bedeutet unprogrammiert) 
/            SUT1..0     10      
/            CKDIV8      0                                  
/
/
/ Frequenz : 1.000.000 Hz   (Grundeinstellung fr die Taktfrequenz der CPU bei Werksauslieferung)
/
/ 
/ Timer0   : Der 8-Bit Timer0 erzeugt im CTC-Modus mit dem im OCR0A-Register
/            hinterlegten Wert das Signal fr den Summer welches an dem 
/            Port-Pin PD5 (OC0B) ausgegeben wird   
/
/
/ Timer1   : Der 16-Bit Timer1 wird benutzt um die spielinternen Taktfrequenzen, 
/            die in den Makros/Konstanten GRUND_GESCHWINDIGKEIT und 
/            SPIEL_GESCHWINDIGKEIT hinterlegt sind zu erzeugen.  
/
/
/ Interrupt: Der EIN_TASTER an Port PD2 wird einerseits zum Starten des Spiels oder zum
/            "Aufwecken" aus dem SLEEP-Modus verwendet, indem diese Taste als Impulsgeber
/            fr den externen Interrupt INT0 verwendet wird. Als Detektierung wird die fallende 
/            Flanke am Port-Pin herangezogen.
/            Andererseits wird der EIN_TASTER auch dazuverwendet, um in die Menu-Funktion 
/            des Spiels zu gelangen, indem die Taste lngere Zeit gedrckt wird. Um das langanhaltende
/            Drcken dieser Taste erkennen zu knnen wird die Interruptfunktionalitt kurzzeitig 
/            abgeschalten.
/
/
/ Display  : Die Ansteuerung des HD44780 kompatiblen Display-Controllers 
/            erfolgt im 4-Bit-Interfacemodus. Dabei sind folgende Port-Pins
/            des C mit den Pins des Display-Moduls verfunden.  
/            PB0->DB4; PB1->DB5; PB2->DB6; PB3->DB7; PA0->E; PA1->RS;    
/
/             
/ Ports    : Taster  : PD0 -> TASTER1
/                      PD1 -> TASTER2
/                      PD3 -> TASTER3
/                      PD4 -> TASTER4
/                      PD2 -> EIN_TASTER
/            LED     : PD0 -> LED1
/                      PD1 -> LED2
/                      PD3 -> LED3
/                      PD4 -> LED4   
/            Summer  : PD5 -> SUMMER
/            Display : PB0 -> DB4   
/                      PB1 -> DB5   
/                      PB2 -> DB6   
/                      PB3 -> DB7 
/                      PA0 -> E  
/                      PA1 -> RS   
/
/           -------------------------------- Beschreibungen --------------------------------------
/ Spiel:	Inhalt dieses Pollin-Spiel-I ist es sich eine zufllig ausgewhlte Reihenfolge von auf-
/			blickenden LEDs zu merken und diese Reihenfolge mit den entsprechenden Tasten, die zu den 
/			LEDs gehren, einzugeben. Dabei startet das Spiel zuerst mit einer LED und wird jeweils
/			um eine weitere Anzeige einer LED erhht sobald man die richtige Reihenfolge des vorge-
/			gebenen Musters eingegeben hat. Bei einer Falscheingabe wird die Reihenfolge wiederholt.
/			Bei richtiger Eingabe der Reihenfolge erhlt man fr jeden richtigen Tastendruck einen
/			Punkt. Bei einer Falscheingabe wird die Punktzahl um einen Zhler reduziert. Die erreichte
/			Punktzahl wird nach jeder Runde angezeigt.
/
/			Bei jedem Aufleuchten einer LED ertnt ein Ton um sich dadurch die Reihenfolge leichter
/			zu merken. Dieser Ton kann ber einen Menpunkt abgeschalten werden. Ebenso kann in einem
/			weiterem Menpunkt die erreichte Hchstpunktzahl angezeigt werden.
/
/			Das Spiel wird durch das Drcken der EIN-Taste begonnen. Wird fr eine lngere Zeit keine
/			Taste mehr gedrckt schaltet sich das Spiel selbstndig ab. Durch lngeres Bettigen der
/			EIN-Taste wird ein Men gestartet von dem durch Drcken des TASTER1 bzw. TASTER2 die 
/			Menpunkte Punkte oder Ton gewhlt werden knnen. Durch erneutes lngeres Drcken
/			der EIN-Taste gelangt man in das entsprechene Untermen und kann dort entweder den Ton
/			aus- bzw. einschalten oder die Hchstpunktzahl betrachten. Durch weiteres Bettigen
/			der EIN-Taste wird das Men verlassen und das Spiel beginnt von vorne.
/
/			Der nachfolgende Quellcode ist fr den Bausatz Pollin-Spiel-I geschrieben welcher auf der
/			Webseite www.pollin.de unter der Best.Nr. 810 148 bezogen werden kann.
/
/
/Achtung:   Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke 
/			genutzt werden darf. Die Firma Pollin Electronic GmbH bernimmt keine Garantie fr die einwand-
/           freie Funktion dieses Programmes und bernimmt dadurch auch keine Haftung fr Folgeschden
/           die aufgrund der Benutzung dieser Daten entstehen. Dieses gilt auch dann, wenn die Firma
/           Pollin Electronic GmbH ber eine solchen Schaden unterrichtet worden ist.  
/  
*/


#include <string.h>
#include <avr/io.h>
#include <inttypes.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include "display.h"
#include <util/delay.h>


#define TASTER1         0             // Port PD0 - meinTasterByte Bit 0
#define TASTER2         1             // Port PD1 - meinTasterByte Bit 1
#define TASTER3         3             // Port PD3 - meinTasterByte Bit 3
#define TASTER4         4             // Port PD4 - meinTasterByte Bit 4
#define EIN_TASTER      2             // Port PD2 - meinTasterByte Bit 2
#define SUMMER          5             // Port PD5 
#define LED1            0             // Port PD0
#define LED2            1             // Port PD1  
#define LED3            3             // Port PD3
#define LED4            4             // Port PD4   
#define DISPLAY_Vo      4             // Port PB4   
#define DISPLAY_POWER   6             // Port PD6   


#define TIMER0_OVR      1
#define TIMER1_OVR      2


#define F_CPU           1000000UL     // Systemtakt 1 MHz

#define BASE_DEZ        10
#define TONHOEHE        0x10


#define STRING_LAENGE_utoa        4
#define STRING_LENGTH_MAX         8
#define GRUND_GESCHWINDIGKEIT     15624UL
#define SPIEL_GESCHWINDIGKEIT     7812UL



/* -------------------------------------------------------------------------------------
   Definition der im Programm verwendeten Displaytextanzeigen
--------------------------------------------------------------------------------------*/ 
#define T_Ton          0
#define T_Punkte       1
#define T_Pollin       2
#define T_Spiel        3
#define T_Menu         4
#define T_Gut          5
#define T_Falsch       6
#define T_nach         7 
#define T_machen       8 
#define T_Muster       9 
#define T_merken       10 
#define T_Stufe        11
#define T_Aus          12
#define T_Ein          13




/* -------------------------------------------------------------------------------------
   Festlegung der Texte die auf dem Display angezeigt werden sollen
--------------------------------------------------------------------------------------*/ 
const char Display_Text0[]     PROGMEM = "Ton   ";
const char Display_Text1[]     PROGMEM = "Punkte";
const char Display_Text2[]     PROGMEM = "Pollin";
const char Display_Text3[]     PROGMEM = "Spiel ";
const char Display_Text4[]     PROGMEM = "Menu";
const char Display_Text5[]     PROGMEM = "Gut";
const char Display_Text6[]     PROGMEM = "Falsch";
const char Display_Text7[]     PROGMEM = "nach-";
const char Display_Text8[]     PROGMEM = "machen";
const char Display_Text9[]     PROGMEM = "Muster";	
const char Display_Text10[]    PROGMEM = "merken";
const char Display_Text11[]    PROGMEM = "Stufe";
const char Display_Text12[]    PROGMEM = "Aus";
const char Display_Text13[]    PROGMEM = "Ein";



const char *Display_Text[] PROGMEM = {	Display_Text0, Display_Text1, Display_Text2, Display_Text3,
									    Display_Text4, Display_Text5, Display_Text6, Display_Text7,
										Display_Text8, Display_Text9, Display_Text10, Display_Text11,
										Display_Text12,Display_Text13 };




/* -------------------------------------------------------------------------------------
   Deklaration und Wertefestlegung der globalen Variablen
--------------------------------------------------------------------------------------*/ 
const uint8_t pgmSpiel_Sequenz[] PROGMEM= {};

char myHelpString[STRING_LENGTH_MAX+1];                                    


uint8_t Spiel_Level;
uint8_t Spiel_Sequenz_Zaehler;
uint8_t Spiel_Sequenz_Zaehler_Offset;


uint8_t Spiel_Phase=0;
uint8_t LED_Ton_Pause=1;
uint8_t Spiel_Ergebnis=1;

  
uint16_t Timer1_Reload_Value   = GRUND_GESCHWINDIGKEIT;


volatile uint8_t Timer1_Zaehler=0;
volatile uint8_t meinInterruptByte=0;


uint8_t  eeTon_Aktiv     EEMEM;
uint16_t eeRecord_Punkte EEMEM;


/* -------------------------------------------------------------------------------------
   INT0-Interrupt-Funktion: - Externer-Interrupt wir durch das Drcken des EIN_TASTER 
                              ausgelst (bei steigender Flanke an Port-Pin PD2) 
   Parameterbergabe: Parameter werden durch globale Variablen bergeben. 
--------------------------------------------------------------------------------------*/ 
ISR(INT0_vect)
{
    uint8_t temp_sreg_INT0;

    temp_sreg_INT0 = SREG;                                     	// Status-Register sichern

		// ====== EIN_TASTER ======
	    TIMSK  |= (1<<TOIE1);                                 	// TOIE1=1 Timer1 Interrupt enable     
		Spiel_Sequenz_Zaehler_Offset=TCNT1;

        GIMSK  &= ~(1<<INT0);           						// INT0=1 Externer Interrupt an Pin PD2 disable     
        TCNT1   = 65535UL-GRUND_GESCHWINDIGKEIT;				// Timer1 Anfangswert fr Overflow-Interrupt setzen
    
        Spiel_Phase=1;
	    Spiel_Level=1;
        Spiel_Sequenz_Zaehler=0;        
		
        Timer1_Zaehler=1;

    SREG = temp_sreg_INT0;            							// Status-Register zurckschreiben
	return;
}



/* -------------------------------------------------------------------------------------
   Timer1-Interrupt-Funktion: 
   Parameterbergabe: Parameter werden duch globale Variablen bergeben. 
--------------------------------------------------------------------------------------*/ 
ISR(TIMER1_OVF_vect)
{
    uint8_t temp_sreg_Timer1;

    temp_sreg_Timer1 = SREG;                                    // Status-Register sichern

    Timer1_Zaehler++;
    meinInterruptByte = (1<<TIMER1_OVR);                       // setzt Bit fr TIMER1_OVR-Erkennung

    TCNT1   = 65535UL-Timer1_Reload_Value;                     // Timer1 Anfangswert fr Overflow-Interrupt setzen

    SREG = temp_sreg_Timer1;                                   // Status-Register zurckschreiben
	return;
}



/* -------------------------------------------------------------------------------------
   Funktion: 			In dieser Funktion wird in der Spielphase 4 die gedrckte Taste
                        mit der vorgegebenen Tastenkombination verglichen. Weiterhin, wird 
                        die Tonhhe fr den beim Tastendruck erzeugten Ton im
                        entsprechendem Register hinterlegt.
   Parameterbergabe:   Input     : Gedrueckte_Taste    bergibt den Wert als Byte welche
													    Taste gedrckt wurde 
					    Return    : -
--------------------------------------------------------------------------------------*/ 
void Funktion_Tastendruck(uint8_t Gedrueckte_Taste)
{
	if(Spiel_Phase==4)
	{
        Timer1_Zaehler=1;
		Spiel_Ergebnis=1;

		if((pgm_read_byte(&pgmSpiel_Sequenz[Spiel_Sequenz_Zaehler+Spiel_Sequenz_Zaehler_Offset]) & 0b00000011)!=(Gedrueckte_Taste & 0b00000011))
 			Spiel_Ergebnis=0;


		Spiel_Sequenz_Zaehler++;


	    if(Spiel_Sequenz_Zaehler==Spiel_Level || Spiel_Ergebnis==0)
        {
			Spiel_Phase=5;
			meinInterruptByte &= ~(1<<TIMER1_OVR);        // lscht Bit fr Timer1_OVR-Erkennung 
        }

	}

    OCR0A=TONHOEHE+Gedrueckte_Taste;					   // hinterlegt die Tonhhe im OCR0A-Register 
    return;
}



/* -------------------------------------------------------------------------------------
   Funktion :			Die Funktion Display_Anzeige() liest entsprechend dem bergebenen Parameter 
						den entsprechenden Text aus dem Flash-Speicher aus und bergibt diesen
						Text der Funktion WriteDisplayString() durch die dieser dann am Display
						angezeigt wird. 
   Parameterbergabe:   Input     : Text_Nr    bergibt die Nummer des zu anzeigenden Textes
						Return    : -
--------------------------------------------------------------------------------------*/ 
void Display_Anzeige(uint8_t Text_Nr)
{		
	strcpy_P(myHelpString,  (const char*) (pgm_read_word(&(Display_Text[Text_Nr]))));
	WriteDisplayString(myHelpString);
    return;
}




/* -------------------------------------------------------------------------------------
   Funktion :			Die Funktion ZahlZuString() stellt eine abgespeckte Version der utoa-Funktion
						der aus der stdlib.h-Bibliotek dar. Hierbei wir eine 16-Bit groe Zahl
                        in einen ASCCII-String umgeformt. 
   Parameterbergabe:   Input     : Val        In dieser Variable wird der umzuwandelnde Zahlenwert bergeben
						Return    : String     Rckgabe des Umgewandelten Zahlenwertes
--------------------------------------------------------------------------------------*/ 
char* ZahlZuString(uint16_t Val, char* String)
{
	uint8_t p;
	uint8_t i = STRING_LAENGE_utoa;
 
	String[i] = '\0';
	
	while(i>0)
    { 
		i--;
		String[i] = '0' + Val % BASE_DEZ;
		Val /= BASE_DEZ;
   
        if(String[i]=='0' && Val==0)
		{ 
			p=i;
			while(p<=STRING_LAENGE_utoa)
			{
				String[p]=String[p+1];
				p++;
			}
   
		}
   
	}

	return String;

}



/* -------------------------------------------------------------------------------------
   Hauptprogramm	:	Pollin-Spiel-I (Beschreibung siehe Einleitung)
   Parameterbergabe:   Input     : -
						Return    : -
--------------------------------------------------------------------------------------*/ 
int main(void)
{
    // ================= Initialisierung ===================

    // ---------- Portpinbelegung und Definition ----------- 

    // ------ Port D (Taster1/2/3/4 und EIN_TASTER) ------
    DDRD  |= ((1<<SUMMER) | (1<<DISPLAY_POWER));                                                     // setzt Port-Pins als Ausgnge fest  
    DDRD  &= ~((1<<TASTER1) | (1<<TASTER2) | (1<<TASTER3) | (1<<TASTER4) | (1<<EIN_TASTER));       // definiert Port-Pins fr TASTER als Eingnge 

    PORTD |=  (1<<DISPLAY_POWER);                                                                     // setze Port-Pin auf High-Pegel 
    PORTD &= ~((1<<TASTER1) | (1<<TASTER2) | (1<<TASTER3) | (1<<TASTER4) | (1<<EIN_TASTER));       // deaktiviere internen Pull-Ups von Port-Pin



    // --------------- Port B ------------- 
    DDRB  |=  (1<<DISPLAY_Vo);                                  // setzt Port-Pin als Ausgnge fest
 	PORTB |=  (1<<DISPLAY_Vo);                                  // setze Port-Pin auf High-Pegel 


    // ------- Interrupt fr EIN_TASTER definieren ---------      
    MCUCR  |= (1<<ISC00) | (1<<ISC01);                         // ISC00=1, ISC01=1 steigende Flanke an INT0 erzeugt einen Interrupt 
    // MCUCR  |= (1<<ISC00);                                      // ISC00=1, ISC01=0 jede Logignderung am Eingang erzeugt einen Interrupt 
    // MCUCR  &= ~((1<<ISC00) | (1<<ISC01));                       // ISC00=0, ISC01=0 Low-Level am Eingang erzeugt einen Interrupt 
    	

    // --------------- Timer0 fr CTC-Modus ----------------    // Timer0 ist ein 8Bit Timer und wird hier im CTC-Modus verwendet
    // TCCR0A: COM0A1/COM0A0/COM0B1/COM0B0/-/-/WGM01/WGM00 
    // TCCR0B: FOC0A/FOC0B/-/-/WGM02/CS02/CS01/CS00
    // 
    // WGM02=0, WGM01=1, WGM00=0 => CTC-Modus aktiviert
    // COM0B1=0 ,COM0B0=0        => OC0B disconnected / Pin-Funktion aktiv
    // CS02=0 , CS01=1 ; CS00=1  => Vorteiler 64      
    TCCR0A = 0b00000010;                                         
    TCCR0B = 0b00000011;   										



    // -------- Timer1 fr interne Zeitmessung ---------        // Timer1 ist ein 16Bit Timer
    TIMSK  |= (1<<TOIE1);                                      // TOIE1=1 Interrupt enable     
    TCCR1B |= (0<<CS12) | (1<<CS11) | (1<<CS10);				// CS12=0, CS11=1, CS10=1 => Vorteiler 64;  1.000.000 Hz : 64 = 15.625 Hz       


 
    sei();														// aktiviert global alle definierten Interrupte
 
	InitDisplay();

 
  
    // ------------ lokale Variablen ------------

    uint8_t meinHilfeByte;
    uint8_t meinTasterByte;										// Temporrer Speicher fr bettigte Taster   

    meinTasterByte = 0b00000000;                                           

    uint8_t  Ton_Aktiv;

	
    if(eeprom_read_byte(&eeTon_Aktiv)==0xFF)
 	{	
		Ton_Aktiv=1;
	    eeprom_write_word(&eeRecord_Punkte, 0); // schreiben

	}
    else
	{
		Ton_Aktiv=eeprom_read_byte(&eeTon_Aktiv);
	}


    uint8_t  Menu_Ebene=0;
    uint8_t  aktuelle_menu_id=0;
    uint16_t Punkte=0;



    while(1)
    {

        // ====== Ton bei TASTER-Druck ======

        if(meinTasterByte & ((1<<TASTER1) | (1<<TASTER2) | (1<<TASTER3) | (1<<TASTER4)))
		{
		    if(Ton_Aktiv==1)
 			   TCCR0A = 0b00010010;   // Ton an, da COM0B1=0 und COM0B0=1 die Pin-Funktion des Port-Ausgangs fr OC0B Toggle aktivieren
 		}
		else
		{
			TCCR0A = 0b00000010;      // Ton aus, da COM0B1=0 und COM0B0=0 die Pin-Funktion fr den CTC-Modus deaktivieren
 		}



        // ====== TASTER1 ======
        if(PIND & (1<<TASTER1))
        {
           if(~meinTasterByte & (1<<TASTER1))       // beim ersten Tastendruck wahr
           {   meinTasterByte |= (1<<TASTER1);       // setzt Bit fr TASTER1

                if(Menu_Ebene==1)
                { 				
					if(aktuelle_menu_id<1)
						aktuelle_menu_id++;
  						
					SetDisplayCursor(2,1);
			        Display_Anzeige(aktuelle_menu_id);
                }


                if(Menu_Ebene==2)    
				{	
                    if(aktuelle_menu_id==0)
					{
						SetDisplayCursor(2,1);

						if(Ton_Aktiv==1)
						{
							Ton_Aktiv=0;
						}
						else
						{
							Ton_Aktiv=1;
						}

						Display_Anzeige(T_Aus + Ton_Aktiv);


					}

				}

               Funktion_Tastendruck(1);
           }
        }
        else
           meinTasterByte &= ~(1<<TASTER1);         // lscht Bit fr TASTER1



        // ====== TASTER2 ======
        if(PIND & (1<<TASTER2))
        {
           if(~meinTasterByte & (1<<TASTER2))       // beim ersten Tastendruck wahr
           {   meinTasterByte |= (1<<TASTER2);      // setzt Bit fr TASTER2

				if(Menu_Ebene==1)
				{ 				
					if(aktuelle_menu_id>0)
						aktuelle_menu_id--;
  						
					SetDisplayCursor(2,1);
			        Display_Anzeige(aktuelle_menu_id);
				}

               Funktion_Tastendruck(2);
           }
        }
        else
           meinTasterByte &= ~(1<<TASTER2);         // lscht Bit fr TASTER2



        // ====== TASTER3 ======
        if(PIND & (1<<TASTER3))
        {
           if(~meinTasterByte & (1<<TASTER3))       // beim ersten Tastendruck wahr
           {   meinTasterByte |= (1<<TASTER3);      // setzt Bit fr TASTER3

               Funktion_Tastendruck(3);
           }
        }
        else
           meinTasterByte &= ~(1<<TASTER3);         // lscht Bit fr TASTER3



        // ====== TASTER4 ======
        if(PIND & (1<<TASTER4))
        {
           if(~meinTasterByte & (1<<TASTER4))       // beim ersten Tastendruck wahr
           {   meinTasterByte |= (1<<TASTER4);      // setzt Bit fr TASTER4

               Funktion_Tastendruck(4);
           }
        }
        else
           meinTasterByte &= ~(1<<TASTER4);         // lscht Bit fr TASTER4


        // === Wartezeit die zum Entprellen ===
		// ===      der Tasten dient        === 
	    _delay_us(5000);							 // --- Warte 5 ms ---



        // ====== Timer1_Zaehler-Auswertung ======
        if(Timer1_Zaehler==0 || Timer1_Zaehler>20)
        {

		    GIMSK |= (1<<INT0);                     // INT0=1 Externer Interrupt an Pin PD2 enable     
  
			if(Timer1_Zaehler>30)
			{
                PORTD &= ~(1<<DISPLAY_POWER);                  // setze Port-Pins auf Low-Pegel 
                PORTB &= ~(1<<DISPLAY_Vo);                     // setze Port-Pins auf Low-Pegel 
                
				SleepDisplay();

				TIMSK  &= ~(1<<TOIE1);                         // TOIE1=1 Timer1-Interrupt disable     


                 
				// ---- Sleep-Mode-Idle aktivieren ----
                MCUCR &= ~((1<<SM1) | (1<<SM0));              // SM1=0, SM0=0 => Sleep-Modus-Idle 
                // MCUCR |= ((1<<SM1) | (1<<SM0));              // SM1=1, SM0=1 => Sleep-Modus-Power-down 
                MCUCR |= (1<<SE);                              // aktiviere den Sleep-Modus 
                asm volatile ("sleep"::);


				PORTD |= (1<<DISPLAY_POWER);                   // setze Port-Pins auf High-Pegel 
				PORTB |= (1<<DISPLAY_Vo);                      // setze Port-Pins auf High-Pegel 

				InitDisplay();

  			    meinInterruptByte &= ~(1<<TIMER1_OVR);         // lscht Bit fr Timer1_OVR 
                Timer1_Zaehler=1;
				Menu_Ebene=0;

			}

			SetDisplayCursor(1,1);
			Display_Anzeige(T_Pollin);
			SetDisplayCursor(2,1);
			Display_Anzeige(T_Spiel);

       }
 
 
 
        if((meinInterruptByte & (1<<TIMER1_OVR)) && Spiel_Phase!=4)       // berprft ob ein Timer1-Overflow-Interrupt ausgelst wurde
		{

			meinInterruptByte &= ~(1<<TIMER1_OVR);                // lscht Bit fr Timer1_OVR 


				// ====== EIN_TASTER ======
				if(PIND & (1<<EIN_TASTER))
				{
					if(~meinTasterByte & (1<<EIN_TASTER))         // beim ersten Tastendruck wahr
					{   meinTasterByte |= (1<<EIN_TASTER);        // setzt Bit fr EIN_TASTER

						ClearDisplay();
						Punkte=0;
                        Spiel_Phase=0;
                       
                        if(Menu_Ebene==2)    
                        {
						   Menu_Ebene=0;
                            if(aktuelle_menu_id==0)
								eeprom_write_byte(&eeTon_Aktiv, Ton_Aktiv); // schreiben
						}
                        else 
							if(Menu_Ebene==1)    
							{
								Menu_Ebene=2;

								if(aktuelle_menu_id==0)
									SetDisplayCursor(1,2);
                                else
									SetDisplayCursor(1,1);


								Display_Anzeige(aktuelle_menu_id);
								SetDisplayCursor(2,1);

								if(aktuelle_menu_id==0)
								{
									if(Ton_Aktiv==1) 
									    Display_Anzeige(T_Ein);
									else
										Display_Anzeige(T_Aus);
								}
                                else 
									WriteDisplayString(ZahlZuString(eeprom_read_word(&eeRecord_Punkte), myHelpString));


							}
							else
							{      
								Menu_Ebene=1;
 							    SetDisplayCursor(1,2);
								Display_Anzeige(T_Menu);
								SetDisplayCursor(2,1);
								Display_Anzeige(aktuelle_menu_id);
							}
 						
					}
				}
				else
				{
					meinTasterByte &= ~(1<<EIN_TASTER);        // lscht Bit fr EIN_TASTER

                    if(Menu_Ebene==0)    
					    GIMSK |= (1<<INT0);                    // INT0=1 externer Interrupt an Pin PD2 enable     
				}



			if(Spiel_Phase==6)
			{
				Spiel_Phase=1;
			}
			
			
			if(Spiel_Phase==5)
			{

				ClearDisplay();
				if(Spiel_Ergebnis==1)
				{
					SetDisplayCursor(1,2);
                    Display_Anzeige(T_Gut);

                    Punkte=Punkte+Spiel_Level;
					Spiel_Level++;
				} 
				else
				{
					SetDisplayCursor(1,1);
                    Display_Anzeige(T_Falsch);

					if(Punkte>0)
						Punkte--;
				} 

				SetDisplayCursor(2,3);
                WriteDisplayString(ZahlZuString(Punkte, myHelpString));


                // Record_Punkte
                if (eeprom_read_word(&eeRecord_Punkte)<Punkte)
				{
				    eeprom_write_word(&eeRecord_Punkte, Punkte); // schreiben
				}
		
				Spiel_Phase=6;
				Spiel_Sequenz_Zaehler=0;
				
			}


			if(Spiel_Phase==3)
			{
                Timer1_Reload_Value=GRUND_GESCHWINDIGKEIT;
				Spiel_Phase=4;
			    Spiel_Sequenz_Zaehler=0;


				ClearDisplay();
				SetDisplayCursor(1,1);
                Display_Anzeige(T_nach);
				SetDisplayCursor(2,1);
				Display_Anzeige(T_machen);
						

	            DDRD  &= ~((1<<TASTER1) | (1<<TASTER2) | (1<<TASTER3) | (1<<TASTER4));       // definiert alle Taster als Eingnge 
                PORTD &= ~((1<<TASTER1) | (1<<TASTER2) | (1<<TASTER3) | (1<<TASTER4));       // deaktiviere internen Pull-Ups von Port-Pin
			}



            if(Spiel_Phase==2)
			{
			    Timer1_Zaehler=1;
					
				if(LED_Ton_Pause==1)
                {	
						ClearDisplay();
						SetDisplayCursor(1,1);
				        Display_Anzeige(T_Muster);
						SetDisplayCursor(2,1);
 				        Display_Anzeige(T_merken);


						PORTD &= ~((1<<LED1) | (1<<LED2) | (1<<LED3) | (1<<LED4));      // LEDs aus
 
                    LED_Ton_Pause=0;
					Timer1_Reload_Value = SPIEL_GESCHWINDIGKEIT/3;

                }
				else 
				{   
					Timer1_Reload_Value = SPIEL_GESCHWINDIGKEIT;

 
                        meinHilfeByte=pgm_read_byte(&pgmSpiel_Sequenz[Spiel_Sequenz_Zaehler+Spiel_Sequenz_Zaehler_Offset]) & 0b00000011;
						
 
                        if(meinHilfeByte==0b00000001)
						{
							PORTD |=  (1<<LED1);                                           // setze Port-Pins auf High-Pegel 

							SetDisplayCursor(2,0);
							WriteDisplayString("1");
						}

                        if(meinHilfeByte==0b00000010)
						{
							PORTD |=  (1<<LED2);                                           // setze Port-Pins auf High-Pegel 

							SetDisplayCursor(2,7);
							WriteDisplayString("2");
						}
	
                        if(meinHilfeByte==0b00000011)
						{
							PORTD |=  (1<<LED3);                                           // setze Port-Pins auf High-Pegel 

							SetDisplayCursor(1,7);
							WriteDisplayString("3");
						}

                        if(meinHilfeByte==0b0000000)
						{
							PORTD |=  (1<<LED4);                                           // setze Port-Pins auf High-Pegel 
					
							SetDisplayCursor(1,0);
							WriteDisplayString("4");
						}


                        LED_Ton_Pause=1;
						Spiel_Sequenz_Zaehler++;


  					    if(Spiel_Sequenz_Zaehler==Spiel_Level)
                        {
							Spiel_Phase=3;
                        }
                    }
			}



            if(Spiel_Phase==1)
			{
				ClearDisplay();
				SetDisplayCursor(1,0);
 				Display_Anzeige(T_Stufe);
 

				SetDisplayCursor(1,6);
                WriteDisplayString(ZahlZuString(Spiel_Level, myHelpString));
						
				DDRD  |= ((1<<LED1) | (1<<LED2) | (1<<LED3) | (1<<LED4));    // setzt Port-Pins als Ausgnge fest
				
				if(Spiel_Level==1)
					Punkte=0;

	            Spiel_Phase=2;
			}

		}
		

	}
}















